#!/bin/bash

LOG_FILE=/var/log/Transmission_install.log
HOST_CONFIG_DIR=/path/transmission/data
HOST_DOWNLOADS_DIR=/data/vfs/mybucket_baeqacmjq
HOST_DATA_DIR=/path/transmission/watch/folder
CONTAINER_NAME=transmission
APP_VERSION=4.0.2
APP_IMAGE=lscr.io/linuxserver/$CONTAINER_NAME:$APP_VERSION

preCheck() {
    
    if ! command -v tee > /dev/null; then
        echo "`date` tee command not found. Please install tee." >> $LOG_FILE
        exit 1
    fi

    if ! command -v docker-compose > /dev/null; then
        echo "`date` docker-compose command not found. Please install docker-compose." | tee -a $LOG_FILE
        exit 1
    fi
    
    if [ -z "$1" ]; then
        echo "`date` Parameter port is missing or empty." | tee -a $LOG_FILE
        exit 1
    fi
    
    PORT=$1
    
    if netstat -tuln | grep ":$PORT " > /dev/null
    then
        echo "`date` Port $PORT is in use" | tee -a $LOG_FILE
        return 1
    else
        echo "`date` Port $PORT is not in use" | tee -a $LOG_FILE
        return 0
    fi
}

install() {
    
    PORT=$1
    echo "`date` start install $CONTAINER_NAME on port $PORT" | tee -a $LOG_FILE
    # Create docker-compose.yml
    echo "version: '3.1'" > docker-compose.yml
    echo "services:" >> docker-compose.yml
    echo "  $CONTAINER_NAME:" >> docker-compose.yml
    echo "    image: $APP_IMAGE" >> docker-compose.yml
    echo "    container_name: $CONTAINER_NAME" >> docker-compose.yml
    echo "    restart: always" >> docker-compose.yml
    echo "    ports:" >> docker-compose.yml
    echo "      - $PORT:9091" >> docker-compose.yml
    echo "      - 51413:51413" >> docker-compose.yml
    echo "      - 51413:51413/udp" >> docker-compose.yml
    echo "    volumes:" >> docker-compose.yml
    echo "      - $HOST_CONFIG_DIR:/config" >> docker-compose.yml
    echo "      - $HOST_DOWNLOADS_DIR:/downloads" >> docker-compose.yml
    echo "      - $HOST_DATA_DIR:/watch" >> docker-compose.yml
    echo "    environment:" >> docker-compose.yml
    echo "      - PUID=1000" >> docker-compose.yml
    echo "      - PGID=1000" >> docker-compose.yml
    echo "      - TZ=Etc/UTC" >> docker-compose.yml
    echo "      - TRANSMISSION_WEB_HOME= #optional" >> docker-compose.yml
    echo "      - USER= #optional" >> docker-compose.yml
    echo "      - PASS= #optional" >> docker-compose.yml
    echo "      - WHITELIST= #optional" >> docker-compose.yml
    echo "      - PEERPORT= #optional" >> docker-compose.yml
    echo "      - HOST_WHITELIST= #optional" >> docker-compose.yml
    
    # Start Transmission
    docker-compose up -d
    upResult=$?
    if [ $upResult -ne 0 ]; then
        echo "`date` install $CONTAINER_NAME failed with exit code $upResult" | tee -a $LOG_FILE
        return 1
    fi
    echo "`date` wait 10 seconds for $CONTAINER_NAME to start" | tee -a $LOG_FILE
    sleep 10
    
    return 0
}

postCheck() {
    echo "`date` start post check for $CONTAINER_NAME installation" | tee -a $LOG_FILE
    # Check Transmission status
    if ! docker-compose ps | grep $CONTAINER_NAME | grep Up > /dev/null; then
        echo "`date` Transmission is not running." | tee -a $LOG_FILE
        return 1
    fi
    return 0
}

# step 1: pre-Check
preCheck $1
preCheckResult=$?
echo "`date` pre check result is $preCheckResult" | tee -a $LOG_FILE
if [ $preCheckResult -eq 0 ]
then
    echo "`date` Pre-check successful." | tee -a $LOG_FILE
    # Step 2: Install Transmission
    install $1
    installResult=$?
    if [ $installResult -eq 0 ]
    then
        echo "`date` Install Transmission successful." | tee -a $LOG_FILE
        # Step 3: Post-check
        postCheck
        postCheckResult=$?
        if [ $postCheckResult -eq 0 ]
        then
            echo "`date` Post-check successful." | tee -a $LOG_FILE
            echo "`date` Transmission installation process completed." | tee -a $LOG_FILE
            exit 0
        else
            echo "`date` Post-check failed." | tee -a $LOG_FILE
            exit 3
        fi
    else
        echo "`date` Install Transmission failed." | tee -a $LOG_FILE
        exit 2
    fi
else
    echo "`date` Pre-check failed." | tee -a $LOG_FILE
    exit 1
fi